@@ -1,4 +1,4 @@
-/* $Id: gucharmap-table.c,v 1.33 2004/02/02 00:48:05 nlevitt Exp $ */
+/* $Id: gucharmap-table.c,v 1.40 2006/02/12 00:07:14 behdad Exp $ */
 /*
  * Copyright (c) 2004 Noah Levitt
  *
@@ -472,6 +472,8 @@ chartable_accessible_factory_class_init (AtkObjectFactoryClass *clazz)
 {
   clazz->create_accessible = chartable_accessible_factory_create_accessible;
   clazz->get_accessible_type = chartable_accessible_factory_get_accessible_type;
+
+  gucharmap_intl_ensure_initialized ();
 }
 
 static GType
@@ -501,6 +503,12 @@ chartable_accessible_factory_get_type (void)
   return t;
 }
 
+static guint
+get_last_cell (GucharmapTable *chartable)
+{
+  return gucharmap_codepoint_list_get_last_index (chartable->codepoint_list);
+}
+
 static void
 set_active_cell (GucharmapTable *chartable,
                  guint           cell)
@@ -511,15 +519,17 @@ set_active_cell (GucharmapTable *chartable,
   chartable->active_cell = cell;
 
   /* update page, if necessary */
-  if (cell - chartable->page_first_cell >= chartable->rows * chartable->cols)
+  if ((gint)cell - chartable->page_first_cell >= chartable->rows * chartable->cols || (gint)cell < chartable->page_first_cell)
     {
       /* move the page_first_cell as far as active_cell has moved */
       gint offset = (gint) chartable->active_cell - (gint) chartable->old_active_cell;
     
-      if ((gint) chartable->old_page_first_cell + offset >= 0)
-        chartable->page_first_cell = chartable->old_page_first_cell + offset;
-      else
+      if ((gint) chartable->old_page_first_cell + offset < 0)
         chartable->page_first_cell = 0;
+      else if ((gint) chartable->old_page_first_cell + offset > get_last_cell (chartable) - (get_last_cell (chartable) % chartable->cols) - chartable->cols * (chartable->rows - 1))
+        chartable->page_first_cell = get_last_cell (chartable) - (get_last_cell (chartable) % chartable->cols) - chartable->cols * (chartable->rows - 1);
+      else
+        chartable->page_first_cell = chartable->old_page_first_cell + offset;
     
       /* round down so that it's a multiple of chartable->cols */
       chartable->page_first_cell -= (chartable->page_first_cell % chartable->cols);
@@ -587,12 +597,6 @@ set_scrollbar_adjustment (GucharmapTable *chartable)
                             chartable->adjustment_changed_handler_id);
 }
 
-static guint
-get_last_cell (GucharmapTable *chartable)
-{
-  return gucharmap_codepoint_list_get_last_index (chartable->codepoint_list);
-}
-
 /* for mouse clicks */
 static gunichar
 get_cell_at_xy (GucharmapTable *chartable, 
@@ -635,12 +639,12 @@ draw_character (GucharmapTable *chartable,
   cell = get_cell_at_rowcol (chartable, row, col);
   wc = gucharmap_codepoint_list_get_char (chartable->codepoint_list, cell);
 
-  if (wc < 0 || wc > UNICHAR_MAX || !gucharmap_unichar_validate (wc) || !gucharmap_unichar_isdefined (wc))
+  if (wc > UNICHAR_MAX || !gucharmap_unichar_validate (wc) || !gucharmap_unichar_isdefined (wc))
     return;
 
-  if (GTK_WIDGET_HAS_FOCUS (chartable->drawing_area) && cell == chartable->active_cell)
+  if (GTK_WIDGET_HAS_FOCUS (chartable->drawing_area) && (gint)cell == chartable->active_cell)
     gc = chartable->drawing_area->style->text_gc[GTK_STATE_SELECTED];
-  else if (cell == chartable->active_cell)
+  else if ((gint)cell == chartable->active_cell)
     gc = chartable->drawing_area->style->text_gc[GTK_STATE_ACTIVE];
   else
     gc = chartable->drawing_area->style->text_gc[GTK_STATE_NORMAL];
@@ -677,11 +681,11 @@ draw_square_bg (GucharmapTable *chartable, gint row, gint col)
 
   gc = gdk_gc_new (GDK_DRAWABLE (chartable->drawing_area->window));
 
-  if (GTK_WIDGET_HAS_FOCUS (chartable->drawing_area) && cell == chartable->active_cell)
+  if (GTK_WIDGET_HAS_FOCUS (chartable->drawing_area) && (gint)cell == chartable->active_cell)
     untinted = chartable->drawing_area->style->base[GTK_STATE_SELECTED];
-  else if (cell == chartable->active_cell)
+  else if ((gint)cell == chartable->active_cell)
     untinted = chartable->drawing_area->style->base[GTK_STATE_ACTIVE];
-  else if (cell > gucharmap_codepoint_list_get_last_index (chartable->codepoint_list))
+  else if ((gint)cell > get_last_cell (chartable))
     untinted = chartable->drawing_area->style->dark[GTK_STATE_NORMAL]; 
   else if (! gucharmap_unichar_validate (wc))
     untinted = chartable->drawing_area->style->fg[GTK_STATE_INSENSITIVE];
@@ -908,7 +912,7 @@ gucharmap_table_redraw (GucharmapTable *chartable,
 
   row_offset = ((gint) chartable->page_first_cell - (gint) chartable->old_page_first_cell) / chartable->cols;
 
-#ifdef G_PLATFORM_WIN32
+#ifdef G_OS_WIN32
 
   if (row_offset != 0)
     {
@@ -919,7 +923,7 @@ gucharmap_table_redraw (GucharmapTable *chartable,
       actives_done = TRUE;
     }
 
-#else /* #ifdef G_PLATFORM_WIN32 */
+#else /* #ifdef G_OS_WIN32 */
 
   if (chartable->codepoint_list_changed 
           || row_offset >= chartable->rows 
@@ -966,20 +970,6 @@ gucharmap_table_redraw (GucharmapTable *chartable,
   chartable->old_active_cell = chartable->active_cell;
 }
 
-#if 0
-void
-logg (const gchar *location,
-      const gchar *message)
-{
-  static GTimer *timer = NULL;
-
-  if (timer == NULL)
-    timer = g_timer_new ();
-
-  g_print ("%7.3f: %36s:    %s\n", g_timer_elapsed (timer, NULL), location, message);
-}
-#endif
-
 /* redraws the screen from the backing pixmap */
 static gint
 expose_event (GtkWidget *widget, 
@@ -1052,9 +1042,9 @@ update_scrollbar_adjustment (GucharmapTable *chartable)
 {
   chartable->adjustment->value = 1.0 * chartable->page_first_cell / chartable->cols; 
   chartable->adjustment->lower = 0.0;
-  chartable->adjustment->upper = 1.0 * gucharmap_codepoint_list_get_last_index (chartable->codepoint_list) / chartable->cols; 
-  chartable->adjustment->step_increment = 2.0; 
-  chartable->adjustment->page_increment = 3.0 * chartable->rows, 
+  chartable->adjustment->upper = 1.0 * ( get_last_cell (chartable) / chartable->cols + 1 );
+  chartable->adjustment->step_increment = 3.0;
+  chartable->adjustment->page_increment = 1.0 * chartable->rows;
   chartable->adjustment->page_size = chartable->rows;
 
   gtk_adjustment_changed (chartable->adjustment);
@@ -1118,7 +1108,7 @@ move_up (GucharmapTable *chartable)
 static void
 move_down (GucharmapTable *chartable)
 {
-  if (chartable->active_cell <= gucharmap_codepoint_list_get_last_index (chartable->codepoint_list) - chartable->cols)
+  if (chartable->active_cell <= get_last_cell (chartable) - chartable->cols)
     set_active_cell (chartable, chartable->active_cell + chartable->cols);
 }
 
@@ -1160,10 +1150,10 @@ move_page_up (GucharmapTable *chartable)
 static void
 move_page_down (GucharmapTable *chartable)
 {
-  if (chartable->active_cell < gucharmap_codepoint_list_get_last_index (chartable->codepoint_list) - chartable->cols * chartable->rows)
+  if (chartable->active_cell < get_last_cell (chartable) - chartable->cols * chartable->rows)
     set_active_cell (chartable, chartable->active_cell + chartable->cols * chartable->rows);
-  else if (chartable->active_cell < gucharmap_codepoint_list_get_last_index (chartable->codepoint_list))
-    set_active_cell (chartable, gucharmap_codepoint_list_get_last_index (chartable->codepoint_list));
+  else if (chartable->active_cell < get_last_cell (chartable))
+    set_active_cell (chartable, get_last_cell (chartable));
 }
 
 static gint
@@ -1259,7 +1249,7 @@ set_top_row (GucharmapTable *chartable,
 {
   gint r, c;
 
-  g_return_if_fail (row >= 0 && row <= gucharmap_codepoint_list_get_last_index (chartable->codepoint_list) / chartable->cols);
+  g_return_if_fail (row >= 0 && row <= get_last_cell (chartable) / chartable->cols);
 
   chartable->old_page_first_cell = chartable->page_first_cell;
   chartable->old_active_cell = chartable->active_cell;
@@ -1267,7 +1257,7 @@ set_top_row (GucharmapTable *chartable,
   chartable->page_first_cell = row * chartable->cols;
 
   /* character is still on the visible page */
-  if (chartable->active_cell - chartable->page_first_cell > 0
+  if (chartable->active_cell - chartable->page_first_cell >= 0
       && chartable->active_cell - chartable->page_first_cell < chartable->rows * chartable->cols)
     return;
 
@@ -1279,8 +1269,8 @@ set_top_row (GucharmapTable *chartable,
     r = 0;
 
   chartable->active_cell = chartable->page_first_cell + r * chartable->cols + c;
-  if (chartable->active_cell > gucharmap_codepoint_list_get_last_index (chartable->codepoint_list))
-    chartable->active_cell = gucharmap_codepoint_list_get_last_index (chartable->codepoint_list);
+  if (chartable->active_cell > get_last_cell (chartable))
+    chartable->active_cell = get_last_cell (chartable);
 
   g_signal_emit (chartable, gucharmap_table_signals[SET_ACTIVE_CHAR], 0, 
                  gucharmap_table_get_active_character (chartable));
@@ -1397,13 +1387,15 @@ motion_notify_event (GtkWidget *widget,
     {
       guint cell = get_cell_at_xy (chartable, MAX (0, event->x), MAX (0, event->y));
 
-      if (cell != chartable->active_cell)
+      if ((gint)cell != chartable->active_cell)
         {
+          gtk_widget_hide (chartable->zoom_window);
           set_active_cell (chartable, cell);
           gucharmap_table_redraw (chartable, FALSE);
         }
 
       place_zoom_window (chartable, event->x_root, event->y_root);
+      gtk_widget_show (chartable->zoom_window);
     }
 
   return FALSE;
@@ -1447,17 +1439,17 @@ mouse_wheel_up (GucharmapTable *chartable)
 static void
 mouse_wheel_down (GucharmapTable *chartable)
 {
-  if ((gint) gucharmap_codepoint_list_get_last_index (chartable->codepoint_list) - chartable->rows * chartable->cols / 2 < 0)
+  if ((gint) get_last_cell (chartable) - chartable->rows * chartable->cols < 0)
     {
       set_top_row (chartable, 0);
     }
-  else if (chartable->page_first_cell < gucharmap_codepoint_list_get_last_index (chartable->codepoint_list) - chartable->rows * chartable->cols / 2)
+  else if (chartable->page_first_cell + chartable->rows * chartable->cols / 2 < get_last_cell (chartable) - chartable->rows * chartable->cols)
     {
       set_top_row (chartable, (chartable->page_first_cell + chartable->rows * chartable->cols / 2) / chartable->cols);
     }
   else 
     {
-      set_top_row (chartable, gucharmap_codepoint_list_get_last_index (chartable->codepoint_list) / chartable->cols);
+      set_top_row (chartable, get_last_cell (chartable) / chartable->cols - (chartable->rows - 1) );
     }
 
   gucharmap_table_redraw (chartable, TRUE);
@@ -1688,7 +1680,7 @@ gucharmap_table_get_type (void)
         NULL,           /* class_data */
         sizeof (GucharmapTable),
         0,              /* n_preallocs */
-        (GInstanceInitFunc) gucharmap_table_init,
+        (GInstanceInitFunc) gucharmap_table_init
       };
 
       gucharmap_table_type = g_type_register_static (GTK_TYPE_HBOX, "GucharmapTable", 
