@@ -1,4 +1,4 @@
-/* $Id: gucharmap-unicode-info.c,v 1.12 2004/03/07 17:28:37 nlevitt Exp $ */
+/* $Id: gucharmap-unicode-info.c,v 1.15 2006/02/12 00:07:14 behdad Exp $ */
 /*
  * Copyright (c) 2004 Noah Levitt
  *
@@ -65,6 +65,8 @@ gucharmap_get_unicode_name (gunichar wc)
 {
   static gchar buf[32];
 
+  gucharmap_intl_ensure_initialized ();
+
   if ((wc >= 0x3400 && wc <= 0x4DB5) 
       || (wc >= 0x4e00 && wc <= 0x9fa5) 
       || (wc >= 0x20000 && wc <= 0x2A6D6))
@@ -115,6 +117,8 @@ gucharmap_get_unicode_name (gunichar wc)
 G_CONST_RETURN gchar *
 gucharmap_get_unicode_category_name (gunichar wc)
 {
+  gucharmap_intl_ensure_initialized ();
+
   switch (gucharmap_unichar_type (wc))
     {
       case G_UNICODE_CONTROL: return _("Other, Control");
@@ -216,7 +220,7 @@ gucharmap_get_unicode_data_name (gunichar uc)
 {
   gint min = 0;
   gint mid;
-  gint max = sizeof (unicode_names) / sizeof (UnicodeName) - 1;
+  gint max = G_N_ELEMENTS(unicode_names) - 1;
 
   if (uc < unicode_names[0].index || uc > unicode_names[max].index)
     return "";
@@ -229,14 +233,14 @@ gucharmap_get_unicode_data_name (gunichar uc)
       else if (uc < unicode_names[mid].index)
         max = mid - 1;
       else
-        return unicode_names[mid].name;
+        return unicode_name_get_name(&unicode_names[mid]);
     }
 
   return NULL;
 }
 
 gint
-gucharmap_get_unicode_data_name_count ()
+gucharmap_get_unicode_data_name_count (void)
 {
   return G_N_ELEMENTS (unicode_names);
 }
@@ -244,7 +248,7 @@ gucharmap_get_unicode_data_name_count ()
 #if ENABLE_UNIHAN
 
 gint
-gucharmap_get_unihan_count ()
+gucharmap_get_unihan_count (void)
 {
   return G_N_ELEMENTS (unihan);
 }
@@ -258,7 +262,8 @@ _get_unihan (gunichar uc)
   static const Unihan *most_recent_result;
   gint min = 0;
   gint mid;
-  gint max = sizeof (unihan) / sizeof (Unihan) - 1;
+  gint max = G_N_ELEMENTS(unihan) - 1;
+
 
   if (uc < unihan[0].index || uc > unihan[max].index)
     return NULL;
@@ -286,76 +291,6 @@ _get_unihan (gunichar uc)
   return NULL;
 }
 
-G_CONST_RETURN gchar * 
-gucharmap_get_unicode_kDefinition (gunichar uc)
-{
-  const Unihan *uh = _get_unihan (uc);
-  if (uh == NULL)
-    return NULL;
-  else
-    return uh->kDefinition;
-}
-
-G_CONST_RETURN gchar * 
-gucharmap_get_unicode_kCantonese (gunichar uc)
-{
-  const Unihan *uh = _get_unihan (uc);
-  if (uh == NULL)
-    return NULL;
-  else
-    return uh->kCantonese;
-}
-
-G_CONST_RETURN gchar * 
-gucharmap_get_unicode_kMandarin (gunichar uc)
-{
-  const Unihan *uh = _get_unihan (uc);
-  if (uh == NULL)
-    return NULL;
-  else
-    return uh->kMandarin;
-}
-
-G_CONST_RETURN gchar * 
-gucharmap_get_unicode_kTang (gunichar uc)
-{
-  const Unihan *uh = _get_unihan (uc);
-  if (uh == NULL)
-    return NULL;
-  else
-    return uh->kTang;
-}
-
-G_CONST_RETURN gchar * 
-gucharmap_get_unicode_kKorean (gunichar uc)
-{
-  const Unihan *uh = _get_unihan (uc);
-  if (uh == NULL)
-    return NULL;
-  else
-    return uh->kKorean;
-}
-
-G_CONST_RETURN gchar * 
-gucharmap_get_unicode_kJapaneseKun (gunichar uc)
-{
-  const Unihan *uh = _get_unihan (uc);
-  if (uh == NULL)
-    return NULL;
-  else
-    return uh->kJapeneseKun;
-}
-
-G_CONST_RETURN gchar * 
-gucharmap_get_unicode_kJapaneseOn (gunichar uc)
-{
-  const Unihan *uh = _get_unihan (uc);
-  if (uh == NULL)
-    return NULL;
-  else
-    return uh->kJapaneseOn;
-}
-
 #else /* #if ENABLE_UNIHAN */
 
 gint
@@ -445,6 +380,8 @@ get_nameslist (gunichar uc)
   return NULL;
 }
 
+gboolean _gucharmap_unicode_has_nameslist_entry (gunichar uc);
+
 gboolean
 _gucharmap_unicode_has_nameslist_entry (gunichar uc)
 {
